; Sprite functions

; equates
	PlayerStateOld	equ lastvar
	PlayerState		equ PlayerStateOld+$01
	PlayerMove		equ PlayerState+$01
	PlayerX			equ	PlayerMove+$01
	PlayerY			equ PlayerX+$01
	DeltaCont		equ PlayerY+$01
	PlayerSprite	equ DeltaCont+$01
	PlayerSize		equ PlayerSprite+$01
	PlayerCorrX		equ PlayerSize+$01
	PlayerCorrY		equ PlayerCorrX+$01
	JumpPointer		equ PlayerCorrY+$01
	WalkPointer		equ JumpPointer+$02
	JumpCounter		equ WalkPointer+$02
	WalkCounter		equ JumpCounter+$01
	Direction		equ WalkCounter+$01
	XBorder			equ	Direction+$01
	SizeByte		equ	XBorder+$01
	PlayerVis		equ	SizeByte+$01
	Intro			equ	PlayerVis+$01
	lastvar2		equ	Intro+$01

	WALK_INIT		equ $FC
	JUMP_INIT		equ $FA

SetupPlayer:
	php
	rep #$30
	stz.b joy_old
	ldx.b base
	stx.b PlayerX
	sep #$20
	stz.b Direction
	stz.b PlayerMove
	jsr ResetPlayer			; clear sprite settings	
	lda #$55
	sta.b PlayerVis
	plp
	rts

LoadSprites
	php
	sep #$20
	lda #$55
	sta.b SizeByte
	
	rep #$20
	stz.b base				; transfer sprite data
	lda.w #spriteset1&$FFFF
	sta.b adr
	lda.w #spriteset1/$10000
	sta.b adr+$02
	plp
	JMP LoadTiles

ResetPlayer
	stz.b PlayerCorrX
	stz.b PlayerCorrY		; clear x,y corr
	ldx.w #JUMP_INIT
	stx.w JumpPointer
	ldx.w #WALK_INIT
	stx.b WalkPointer		; reset pointers
	stz.b PlayerState
	lda #$01
	sta.b JumpCounter
	stz.b PlayerVis
	sta.b WalkCounter		; reset counters
	lda #$44
	sta.b PlayerSprite
	stz.b PlayerSize		; set stand sprite
	rts

PlayerWalk
	lda.b PlayerMove
	BEQ w_finish
	
	dec.b WalkCounter
	BNE	w_finish			; new sprite ?
	
	lda.b WalkPointer
	clc
	adc #$04
	cmp #$18				; restart loop ?
	BNE w_norestart
	lda #$00
w_norestart
	sta.b WalkPointer
	ldx.b WalkPointer		; New Pointer
	ldy.w walktable+$01,X
	sty.b PlayerSprite		; set sprite, size (16bit)
	lda.w walktable+$03,X
	sta.b PlayerCorrX		; Set correction
	lda.w walktable,X
	sta.b WalkCounter		; New Counter 	
	
w_finish
	rts	

PlayerUpdate
	lda.b PlayerState
	and #$85
	BEQ PlayerWalk
	and #$05
	BNE i_SetupSprite

JumpUpdate
	dec.b JumpCounter
	BNE	i_SetupSprite
							; new sprite
	lda.b JumpPointer
	clc
	adc #$06
	sta.b JumpPointer		; New Pointer
	ldx.b JumpPointer
	
	ldy.w jumptable+$02,X
	sty.b PlayerCorrX		; set x & y (16bit)
	ldy.w jumptable+$04,X
	sty.b DeltaCont			; set delta & sprite (16bit)
	
	lda.w jumptable,X
	BEQ i_Done				; Finished ?
	sta.b JumpCounter		; New counter

	lda.w jumptable+$01,X
	sta.b PlayerSize		; Set size byte
		
i_SetupSprite	
	lda.b DeltaCont
	clc
	adc.b PlayerY
	sta.b PlayerY
	cmp #$e0
	bcs i_Die
i_ret	
	rts
i_Done
	lda #$01
	sta.b PlayerState
	BRA FallSetup
i_Die
	lda.b Intro
	BNE i_ret
	JMP Die

DisplayPlayer
	ldy.w #$0100			; update size
	sty $2102
	lda.b PlayerSize
	ora #$50
	sta $2104
	
	ldy.w #$8000			; update sprite #00
	sty $2102
	lda.b PlayerMove
	and.b XBorder
	clc
	adc.b PlayerX
	sta.b PlayerX
	clc
	adc.b PlayerCorrX
	tax
	sta $2104				; set x
	lda.b PlayerY
	clc
	adc.b PlayerCorrY
	tay
	sta $2104				; set y
	lda.b PlayerSprite
	sta $2104
	lda.b Direction
	ora #$34
	sta $2104				; 0:adr#9, 1-3:pal, 4-5:prio, 6-7:h,v flip
	lda.b PlayerSize
	BNE dp_Finish
	
	txa					; update sprite #01
	sta $2104
	tya
	clc
	adc #$10
	sta $2104
	lda.b PlayerSprite
	adc #$02
	sta $2104
	lda.b Direction
	ora #$34
	sta $2104
dp_Finish
	rts

FallSetup
	lda.b PlayerState
	and #$01
	BEQ fs_Finish
	ldx.w falltable
	stx.b PlayerCorrX			; x,y corrections
	ldx.w falltable+$02
	stx.b PlayerSprite			; sprite, size
	lda.w falltable+$04
	sta.b DeltaCont
	rts
fs_Finish
	JMP ResetPlayer

KneeSetup
	lda.b joy_hi
	and #$04
	sta.b PlayerState
	BEQ ks_finish
	ldx.w kneetable
	stx.b PlayerCorrX			; x,y corrections
	ldx.w kneetable+$02
	stx.b PlayerSprite			; sprite, size
	stz.b DeltaCont
	rts
ks_finish
	JMP ResetPlayer

TableSetup
	php
	sep #$10
	rep #$20
	lda.b (adr)
	sta.b PlayerCorrX
	ldy #$02
	lda (adr),Y
	sta.b PlayerSprite
	ldy #$04
	lda (adr),Y
	tax
	stx.b DeltaCont
	plp
	rts

SpriteSetup						; Enable/Disable Sprite: adr:yyxx, base:sntt
	php							; where n=number,t=tile adr, s=AF.S
	sep #$30					; A=activate, S=size bit
	lda.b base+$01				; calculate size byte
	and #$0F
	tax
	lda #$01
	cpx #$00
	beq ss_gotbit
ss_bitloop
	asl
	asl
	dex
	bne ss_bitloop				; get corr. bit
ss_gotbit
	ldy.b base+$01
	BMI ss_deact
	trb.b SizeByte 
	BRA ss_cont
ss_deact
	tsb.b SizeByte
ss_cont
	asl
	tax
	tya
	and #$10
	BEQ ss_desize
	txa
	tsb.b SizeByte
	BRA ss_cont2
ss_desize
	txa
	trb.b SizeByte
ss_cont2						; finally got the damn byte
	rep #$11
	ldx.w #$0100
	stx $2102
	
	lda.b PlayerSize
	ora #$50
	ora.b PlayerVis
	sta $2104
	lda.b SizeByte
	sta $2104			; set new size
	lda #$80
	xba
	lda.b base+$01
	and #$0F
	asl
	adc.b #$08
	tax
	stx $2102			; set adress
	lda.b adr
	sta $2104
	lda.b adr+$01
	sta $2104
	lda.b base
	sta $2104
	lda.b base+$01
	and #$40
	ora #$36
	sta $2104			; setup sprite data
	plp
	rts

HideSprites
	php
	rep #$30
	lda.w #$0100
	sta $2102
	sep #$20
	lda #$55
	sta $2104
	sta $2104
	sta $2104
	sta.b PlayerVis
	plp
	rts
